/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/* Generated SBE (Simple Binary Encoding) message codec */
#ifndef _APACHE_THRIFT_SBE_LOGGINGCONTEXT_CXX_H_
#define _APACHE_THRIFT_SBE_LOGGINGCONTEXT_CXX_H_

#if __cplusplus >= 201103L
#define SBE_CONSTEXPR constexpr
#define SBE_NOEXCEPT noexcept
#else
#define SBE_CONSTEXPR
#define SBE_NOEXCEPT
#endif

#if __cplusplus >= 201703L
#include <string_view>
#define SBE_NODISCARD [[nodiscard]]
#else
#define SBE_NODISCARD
#endif

#if !defined(__STDC_LIMIT_MACROS)
#define __STDC_LIMIT_MACROS 1
#endif

#include <cstdint>
#include <cstring>
#include <iomanip>
#include <limits>
#include <ostream>
#include <sstream>
#include <stdexcept>
#include <string>
#include <tuple>
#include <vector>

#if defined(WIN32) || defined(_WIN32)
#define SBE_BIG_ENDIAN_ENCODE_16(v) _byteswap_ushort(v)
#define SBE_BIG_ENDIAN_ENCODE_32(v) _byteswap_ulong(v)
#define SBE_BIG_ENDIAN_ENCODE_64(v) _byteswap_uint64(v)
#define SBE_LITTLE_ENDIAN_ENCODE_16(v) (v)
#define SBE_LITTLE_ENDIAN_ENCODE_32(v) (v)
#define SBE_LITTLE_ENDIAN_ENCODE_64(v) (v)
#elif __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
#define SBE_BIG_ENDIAN_ENCODE_16(v) __builtin_bswap16(v)
#define SBE_BIG_ENDIAN_ENCODE_32(v) __builtin_bswap32(v)
#define SBE_BIG_ENDIAN_ENCODE_64(v) __builtin_bswap64(v)
#define SBE_LITTLE_ENDIAN_ENCODE_16(v) (v)
#define SBE_LITTLE_ENDIAN_ENCODE_32(v) (v)
#define SBE_LITTLE_ENDIAN_ENCODE_64(v) (v)
#elif __BYTE_ORDER__ == __ORDER_BIG_ENDIAN__
#define SBE_LITTLE_ENDIAN_ENCODE_16(v) __builtin_bswap16(v)
#define SBE_LITTLE_ENDIAN_ENCODE_32(v) __builtin_bswap32(v)
#define SBE_LITTLE_ENDIAN_ENCODE_64(v) __builtin_bswap64(v)
#define SBE_BIG_ENDIAN_ENCODE_16(v) (v)
#define SBE_BIG_ENDIAN_ENCODE_32(v) (v)
#define SBE_BIG_ENDIAN_ENCODE_64(v) (v)
#else
#error \
    "Byte Ordering of platform not determined. Set __BYTE_ORDER__ manually before including this file."
#endif

#if !defined(SBE_BOUNDS_CHECK_EXPECT)
#if defined(SBE_NO_BOUNDS_CHECK)
#define SBE_BOUNDS_CHECK_EXPECT(exp, c) (false)
#elif defined(_MSC_VER)
#define SBE_BOUNDS_CHECK_EXPECT(exp, c) (exp)
#else
#define SBE_BOUNDS_CHECK_EXPECT(exp, c) (__builtin_expect(exp, c))
#endif

#endif

#define SBE_FLOAT_NAN std::numeric_limits<float>::quiet_NaN()
#define SBE_DOUBLE_NAN std::numeric_limits<double>::quiet_NaN()
#define SBE_NULLVALUE_INT8 (std::numeric_limits<std::int8_t>::min)()
#define SBE_NULLVALUE_INT16 (std::numeric_limits<std::int16_t>::min)()
#define SBE_NULLVALUE_INT32 (std::numeric_limits<std::int32_t>::min)()
#define SBE_NULLVALUE_INT64 (std::numeric_limits<std::int64_t>::min)()
#define SBE_NULLVALUE_UINT8 (std::numeric_limits<std::uint8_t>::max)()
#define SBE_NULLVALUE_UINT16 (std::numeric_limits<std::uint16_t>::max)()
#define SBE_NULLVALUE_UINT32 (std::numeric_limits<std::uint32_t>::max)()
#define SBE_NULLVALUE_UINT64 (std::numeric_limits<std::uint64_t>::max)()

#include "BooleanType.h"
#include "CodecConfig.h"
#include "CompressionAlgorithm.h"
#include "CompressionConfig.h"
#include "ErrorBlame.h"
#include "ErrorClassification.h"
#include "ErrorKind.h"
#include "ErrorSafety.h"
#include "FdMetadata.h"
#include "GroupSizeEncoding.h"
#include "MessageHeader.h"
#include "MetadataType.h"
#include "PayloadExceptionMetadata.h"
#include "ProtocolId.h"
#include "QueueMetadata.h"
#include "RpcKind.h"
#include "RpcPriority.h"
#include "VarDataEncoding.h"
#include "VarStringEncoding.h"

namespace apache {
namespace thrift {
namespace sbe {

class LoggingContext {
 private:
  /**
   * The states in which a encoder/decoder/codec can live.
   *
   * <p>The state machine diagram below, encoded in the dot language, describes
   * the valid state transitions according to the order in which fields may be
   * accessed safely. Tools such as PlantUML and Graphviz can render it.
   *
   * <pre>{@code
   *   digraph G {
   *       NOT_WRAPPED -> V0_BLOCK [label="  wrap(version=0)  "];
   *       V0_BLOCK -> V0_BLOCK [label="  logSampleRatio(?)  "];
   *       V0_BLOCK -> V0_BLOCK [label="  logErrorSampleRatio(?)  "];
   *       V0_BLOCK -> V0_BLOCK [label="  requestIdLength()  "];
   *       V0_BLOCK -> V0_REQUESTID_DONE [label="  requestId(?)  "];
   *       V0_REQUESTID_DONE -> V0_REQUESTID_DONE [label=" routingTargetLength()
   * "]; V0_REQUESTID_DONE -> V0_ROUTINGTARGET_DONE [label="  routingTarget(?)
   * "];
   *   }
   * }</pre>
   */
  enum class CodecState {
    NOT_WRAPPED = 0,
    V0_BLOCK = 1,
    V0_REQUESTID_DONE = 2,
    V0_ROUTINGTARGET_DONE = 3,
  };

  static const std::string STATE_NAME_LOOKUP[4];
  static const std::string STATE_TRANSITIONS_LOOKUP[4];

  static std::string codecStateName(CodecState state) {
    return STATE_NAME_LOOKUP[static_cast<int>(state)];
  }

  static std::string codecStateTransitions(CodecState state) {
    return STATE_TRANSITIONS_LOOKUP[static_cast<int>(state)];
  }

  char* m_buffer = nullptr;
  std::uint64_t m_bufferLength = 0;
  std::uint64_t m_offset = 0;
  std::uint64_t m_position = 0;
  std::uint64_t m_actingBlockLength = 0;
  std::uint64_t m_actingVersion = 0;
  CodecState m_codecState = CodecState::NOT_WRAPPED;

  CodecState codecState() const { return m_codecState; }

  CodecState* codecStatePtr() { return &m_codecState; }

  void codecState(CodecState newState) { m_codecState = newState; }

  inline std::uint64_t* sbePositionPtr() SBE_NOEXCEPT { return &m_position; }

 public:
  static const std::uint16_t SBE_BLOCK_LENGTH = static_cast<std::uint16_t>(8);
  static const std::uint16_t SBE_TEMPLATE_ID = static_cast<std::uint16_t>(6);
  static const std::uint16_t SBE_SCHEMA_ID = static_cast<std::uint16_t>(1);
  static const std::uint16_t SBE_SCHEMA_VERSION = static_cast<std::uint16_t>(0);
  static constexpr const char* SBE_SEMANTIC_VERSION = "5.2";

  enum MetaAttribute { EPOCH, TIME_UNIT, SEMANTIC_TYPE, PRESENCE };

  union sbe_float_as_uint_u {
    float fp_value;
    std::uint32_t uint_value;
  };

  union sbe_double_as_uint_u {
    double fp_value;
    std::uint64_t uint_value;
  };

  using messageHeader = MessageHeader;

  class AccessOrderError : public std::logic_error {
   public:
    explicit AccessOrderError(const std::string& msg) : std::logic_error(msg) {}
  };

  LoggingContext() = default;

  LoggingContext(
      char* buffer,
      const std::uint64_t offset,
      const std::uint64_t bufferLength,
      const std::uint64_t actingBlockLength,
      const std::uint64_t actingVersion,
      CodecState codecState)
      : m_buffer(buffer),
        m_bufferLength(bufferLength),
        m_offset(offset),
        m_position(sbeCheckPosition(offset + actingBlockLength)),
        m_actingBlockLength(actingBlockLength),
        m_actingVersion(actingVersion),
        m_codecState(codecState) {}

  LoggingContext(
      char* buffer,
      const std::uint64_t offset,
      const std::uint64_t bufferLength,
      const std::uint64_t actingBlockLength,
      const std::uint64_t actingVersion)
      : m_buffer(buffer),
        m_bufferLength(bufferLength),
        m_offset(offset),
        m_position(sbeCheckPosition(offset + actingBlockLength)),
        m_actingBlockLength(actingBlockLength),
        m_actingVersion(actingVersion) {}

  LoggingContext(char* buffer, const std::uint64_t bufferLength)
      : LoggingContext(
            buffer, 0, bufferLength, sbeBlockLength(), sbeSchemaVersion()) {}

  LoggingContext(
      char* buffer,
      const std::uint64_t bufferLength,
      const std::uint64_t actingBlockLength,
      const std::uint64_t actingVersion)
      : LoggingContext(
            buffer, 0, bufferLength, actingBlockLength, actingVersion) {}

  SBE_NODISCARD static SBE_CONSTEXPR std::uint16_t sbeBlockLength()
      SBE_NOEXCEPT {
    return static_cast<std::uint16_t>(8);
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::uint64_t sbeBlockAndHeaderLength()
      SBE_NOEXCEPT {
    return messageHeader::encodedLength() + sbeBlockLength();
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::uint16_t sbeTemplateId()
      SBE_NOEXCEPT {
    return static_cast<std::uint16_t>(6);
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::uint16_t sbeSchemaId() SBE_NOEXCEPT {
    return static_cast<std::uint16_t>(1);
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::uint16_t sbeSchemaVersion()
      SBE_NOEXCEPT {
    return static_cast<std::uint16_t>(0);
  }

  SBE_NODISCARD static const char* sbeSemanticVersion() SBE_NOEXCEPT {
    return "5.2";
  }

  SBE_NODISCARD static SBE_CONSTEXPR const char* sbeSemanticType()
      SBE_NOEXCEPT {
    return "";
  }

  SBE_NODISCARD std::uint64_t offset() const SBE_NOEXCEPT { return m_offset; }

  LoggingContext& wrapForEncode(
      char* buffer,
      const std::uint64_t offset,
      const std::uint64_t bufferLength) {
    m_buffer = buffer;
    m_bufferLength = bufferLength;
    m_offset = offset;
    m_actingBlockLength = sbeBlockLength();
    m_actingVersion = sbeSchemaVersion();
    m_position = sbeCheckPosition(m_offset + m_actingBlockLength);
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    codecState(CodecState::V0_BLOCK);
#endif
    return *this;
  }

  LoggingContext& wrapAndApplyHeader(
      char* buffer,
      const std::uint64_t offset,
      const std::uint64_t bufferLength) {
    messageHeader hdr(buffer, offset, bufferLength, sbeSchemaVersion());

    hdr.blockLength(sbeBlockLength())
        .templateId(sbeTemplateId())
        .schemaId(sbeSchemaId())
        .version(sbeSchemaVersion());

    m_buffer = buffer;
    m_bufferLength = bufferLength;
    m_offset = offset + messageHeader::encodedLength();
    m_actingBlockLength = sbeBlockLength();
    m_actingVersion = sbeSchemaVersion();
    m_position = sbeCheckPosition(m_offset + m_actingBlockLength);
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    codecState(CodecState::V0_BLOCK);
#endif
    return *this;
  }

  LoggingContext& wrapForDecode(
      char* buffer,
      const std::uint64_t offset,
      const std::uint64_t actingBlockLength,
      const std::uint64_t actingVersion,
      const std::uint64_t bufferLength) {
    m_buffer = buffer;
    m_bufferLength = bufferLength;
    m_offset = offset;
    m_actingBlockLength = actingBlockLength;
    m_actingVersion = actingVersion;
    m_position = sbeCheckPosition(m_offset + m_actingBlockLength);
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    codecState(CodecState::V0_BLOCK);
#endif
    return *this;
  }

  LoggingContext& sbeRewind() {
    return wrapForDecode(
        m_buffer,
        m_offset,
        m_actingBlockLength,
        m_actingVersion,
        m_bufferLength);
  }

  SBE_NODISCARD std::uint64_t sbePosition() const SBE_NOEXCEPT {
    return m_position;
  }

  // NOLINTNEXTLINE(readability-convert-member-functions-to-static)
  std::uint64_t sbeCheckPosition(const std::uint64_t position) {
    if (SBE_BOUNDS_CHECK_EXPECT((position > m_bufferLength), false)) {
      throw std::runtime_error("buffer too short [E100]");
    }
    return position;
  }

  void sbePosition(const std::uint64_t position) {
    m_position = sbeCheckPosition(position);
  }

  SBE_NODISCARD std::uint64_t encodedLength() const SBE_NOEXCEPT {
    return sbePosition() - m_offset;
  }

  SBE_NODISCARD std::uint64_t decodeLength() const {
    LoggingContext skipper(
        m_buffer,
        m_offset,
        m_bufferLength,
        sbeBlockLength(),
        m_actingVersion,
        m_codecState);
    skipper.skip();
    return skipper.encodedLength();
  }

  SBE_NODISCARD const char* buffer() const SBE_NOEXCEPT { return m_buffer; }

  SBE_NODISCARD char* buffer() SBE_NOEXCEPT { return m_buffer; }

  SBE_NODISCARD std::uint64_t bufferLength() const SBE_NOEXCEPT {
    return m_bufferLength;
  }

  SBE_NODISCARD std::uint64_t actingVersion() const SBE_NOEXCEPT {
    return m_actingVersion;
  }

  void checkEncodingIsComplete() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    switch (m_codecState) {
      case CodecState::V0_ROUTINGTARGET_DONE:
        return;
      default:
        throw AccessOrderError(
            std::string("Not fully encoded, current state: ") +
            codecStateName(m_codecState) +
            ", allowed transitions: " + codecStateTransitions(m_codecState));
    }
#endif
  }

  SBE_NODISCARD static const char* logSampleRatioMetaAttribute(
      const MetaAttribute metaAttribute) SBE_NOEXCEPT {
    switch (metaAttribute) {
      case MetaAttribute::PRESENCE:
        return "required";
      default:
        return "";
    }
  }

  static SBE_CONSTEXPR std::uint16_t logSampleRatioId() SBE_NOEXCEPT {
    return 1;
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::uint64_t logSampleRatioSinceVersion()
      SBE_NOEXCEPT {
    return 0;
  }

  SBE_NODISCARD bool logSampleRatioInActingVersion() SBE_NOEXCEPT {
    return true;
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::size_t logSampleRatioEncodingOffset()
      SBE_NOEXCEPT {
    return 0;
  }

 private:
  void onLogSampleRatioAccessed() const {
    if (codecState() == CodecState::NOT_WRAPPED) {
      throw AccessOrderError(
          std::string("Illegal field access order. ") +
          "Cannot access field \"logSampleRatio\" in state: " +
          codecStateName(codecState()) +
          ". Expected one of these transitions: [" +
          codecStateTransitions(codecState()) +
          "]. Please see the diagram in the docs of the enum LoggingContext::CodecState.");
    }
  }

 public:
  static SBE_CONSTEXPR std::int32_t logSampleRatioNullValue() SBE_NOEXCEPT {
    return SBE_NULLVALUE_INT32;
  }

  static SBE_CONSTEXPR std::int32_t logSampleRatioMinValue() SBE_NOEXCEPT {
    return INT32_C(-2147483647);
  }

  static SBE_CONSTEXPR std::int32_t logSampleRatioMaxValue() SBE_NOEXCEPT {
    return INT32_C(2147483647);
  }

  static SBE_CONSTEXPR std::size_t logSampleRatioEncodingLength() SBE_NOEXCEPT {
    return 4;
  }

  SBE_NODISCARD std::int32_t logSampleRatio() const {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onLogSampleRatioAccessed();
#endif
    std::int32_t val;
    std::memcpy(&val, m_buffer + m_offset + 0, sizeof(std::int32_t));
    return SBE_LITTLE_ENDIAN_ENCODE_32(val);
  }

  LoggingContext& logSampleRatio(const std::int32_t value) {
    std::int32_t val = SBE_LITTLE_ENDIAN_ENCODE_32(value);
    std::memcpy(m_buffer + m_offset + 0, &val, sizeof(std::int32_t));
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onLogSampleRatioAccessed();
#endif
    return *this;
  }

  SBE_NODISCARD static const char* logErrorSampleRatioMetaAttribute(
      const MetaAttribute metaAttribute) SBE_NOEXCEPT {
    switch (metaAttribute) {
      case MetaAttribute::PRESENCE:
        return "required";
      default:
        return "";
    }
  }

  static SBE_CONSTEXPR std::uint16_t logErrorSampleRatioId() SBE_NOEXCEPT {
    return 2;
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::uint64_t
  logErrorSampleRatioSinceVersion() SBE_NOEXCEPT {
    return 0;
  }

  SBE_NODISCARD bool logErrorSampleRatioInActingVersion() SBE_NOEXCEPT {
    return true;
  }

  SBE_NODISCARD static SBE_CONSTEXPR std::size_t
  logErrorSampleRatioEncodingOffset() SBE_NOEXCEPT {
    return 4;
  }

 private:
  void onLogErrorSampleRatioAccessed() const {
    if (codecState() == CodecState::NOT_WRAPPED) {
      throw AccessOrderError(
          std::string("Illegal field access order. ") +
          "Cannot access field \"logErrorSampleRatio\" in state: " +
          codecStateName(codecState()) +
          ". Expected one of these transitions: [" +
          codecStateTransitions(codecState()) +
          "]. Please see the diagram in the docs of the enum LoggingContext::CodecState.");
    }
  }

 public:
  static SBE_CONSTEXPR std::int32_t logErrorSampleRatioNullValue()
      SBE_NOEXCEPT {
    return SBE_NULLVALUE_INT32;
  }

  static SBE_CONSTEXPR std::int32_t logErrorSampleRatioMinValue() SBE_NOEXCEPT {
    return INT32_C(-2147483647);
  }

  static SBE_CONSTEXPR std::int32_t logErrorSampleRatioMaxValue() SBE_NOEXCEPT {
    return INT32_C(2147483647);
  }

  static SBE_CONSTEXPR std::size_t logErrorSampleRatioEncodingLength()
      SBE_NOEXCEPT {
    return 4;
  }

  SBE_NODISCARD std::int32_t logErrorSampleRatio() const {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onLogErrorSampleRatioAccessed();
#endif
    std::int32_t val;
    std::memcpy(&val, m_buffer + m_offset + 4, sizeof(std::int32_t));
    return SBE_LITTLE_ENDIAN_ENCODE_32(val);
  }

  LoggingContext& logErrorSampleRatio(const std::int32_t value) {
    std::int32_t val = SBE_LITTLE_ENDIAN_ENCODE_32(value);
    std::memcpy(m_buffer + m_offset + 4, &val, sizeof(std::int32_t));
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onLogErrorSampleRatioAccessed();
#endif
    return *this;
  }

  SBE_NODISCARD static const char* requestIdMetaAttribute(
      const MetaAttribute metaAttribute) SBE_NOEXCEPT {
    switch (metaAttribute) {
      case MetaAttribute::PRESENCE:
        return "required";
      default:
        return "";
    }
  }

  static const char* requestIdCharacterEncoding() SBE_NOEXCEPT {
    return "UTF-8";
  }

  static SBE_CONSTEXPR std::uint64_t requestIdSinceVersion() SBE_NOEXCEPT {
    return 0;
  }

  bool requestIdInActingVersion() SBE_NOEXCEPT { return true; }

  static SBE_CONSTEXPR std::uint16_t requestIdId() SBE_NOEXCEPT { return 3; }

  static SBE_CONSTEXPR std::uint64_t requestIdHeaderLength() SBE_NOEXCEPT {
    return 4;
  }

 private:
  void onRequestIdLengthAccessed() const {
    switch (codecState()) {
      case CodecState::V0_BLOCK:
        break;
      default:
        throw AccessOrderError(
            std::string("Illegal field access order. ") +
            "Cannot decode length of var data \"requestId\" in state: " +
            codecStateName(codecState()) +
            ". Expected one of these transitions: [" +
            codecStateTransitions(codecState()) +
            "]. Please see the diagram in the docs of the enum LoggingContext::CodecState.");
    }
  }

 public:
  SBE_NODISCARD std::uint32_t requestIdLength() const {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdLengthAccessed();
#endif
    std::uint32_t length;
    std::memcpy(&length, m_buffer + sbePosition(), sizeof(std::uint32_t));
    return SBE_LITTLE_ENDIAN_ENCODE_32(length);
  }

 private:
  void onRequestIdAccessed() {
    switch (codecState()) {
      case CodecState::V0_BLOCK:
        codecState(CodecState::V0_REQUESTID_DONE);
        break;
      default:
        throw AccessOrderError(
            std::string("Illegal field access order. ") +
            "Cannot access field \"requestId\" in state: " +
            codecStateName(codecState()) +
            ". Expected one of these transitions: [" +
            codecStateTransitions(codecState()) +
            "]. Please see the diagram in the docs of the enum LoggingContext::CodecState.");
    }
  }

 public:
  std::uint64_t skipRequestId() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    sbePosition(lengthPosition + lengthOfLengthField + dataLength);
    return dataLength;
  }

  SBE_NODISCARD const char* requestId() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + sbePosition(), sizeof(std::uint32_t));
    const char* fieldPtr = m_buffer + sbePosition() + 4;
    sbePosition(
        sbePosition() + 4 + SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue));
    return fieldPtr;
  }

  std::uint64_t getRequestId(char* dst, const std::uint64_t length) {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    sbePosition(lengthPosition + lengthOfLengthField);
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    std::uint64_t bytesToCopy = length < dataLength ? length : dataLength;
    std::uint64_t pos = sbePosition();
    sbePosition(pos + dataLength);
    std::memcpy(dst, m_buffer + pos, static_cast<std::size_t>(bytesToCopy));
    return bytesToCopy;
  }

  LoggingContext& putRequestId(const char* src, const std::uint32_t length) {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    std::uint32_t lengthFieldValue = SBE_LITTLE_ENDIAN_ENCODE_32(length);
    sbePosition(lengthPosition + lengthOfLengthField);
    std::memcpy(
        m_buffer + lengthPosition, &lengthFieldValue, sizeof(std::uint32_t));
    if (length != std::uint32_t(0)) {
      std::uint64_t pos = sbePosition();
      sbePosition(pos + length);
      std::memcpy(m_buffer + pos, src, length);
    }
    return *this;
  }

  std::string getRequestIdAsString() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    sbePosition(lengthPosition + lengthOfLengthField);
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    std::uint64_t pos = sbePosition();
    const std::string result(m_buffer + pos, dataLength);
    sbePosition(pos + dataLength);
    return result;
  }

  std::string getRequestIdAsJsonEscapedString() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::ostringstream oss;
    std::string s = getRequestIdAsString();

    for (const auto c : s) {
      switch (c) {
        case '"':
          oss << "\\\"";
          break;
        case '\\':
          oss << "\\\\";
          break;
        case '\b':
          oss << "\\b";
          break;
        case '\f':
          oss << "\\f";
          break;
        case '\n':
          oss << "\\n";
          break;
        case '\r':
          oss << "\\r";
          break;
        case '\t':
          oss << "\\t";
          break;

        default:
          if ('\x00' <= c && c <= '\x1f') {
            oss << "\\u" << std::hex << std::setw(4) << std::setfill('0')
                << (int)(c);
          } else {
            oss << c;
          }
      }
    }

    return oss.str();
  }

#if __cplusplus >= 201703L
  std::string_view getRequestIdAsStringView() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRequestIdAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    sbePosition(lengthPosition + lengthOfLengthField);
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    std::uint64_t pos = sbePosition();
    const std::string_view result(m_buffer + pos, dataLength);
    sbePosition(pos + dataLength);
    return result;
  }
#endif

  LoggingContext& putRequestId(const std::string& str) {
    if (str.length() > 1073741824) {
      throw std::runtime_error("std::string too long for length type [E109]");
    }
    return putRequestId(str.data(), static_cast<std::uint32_t>(str.length()));
  }

#if __cplusplus >= 201703L
  LoggingContext& putRequestId(const std::string_view str) {
    if (str.length() > 1073741824) {
      throw std::runtime_error("std::string too long for length type [E109]");
    }
    return putRequestId(str.data(), static_cast<std::uint32_t>(str.length()));
  }
#endif

  SBE_NODISCARD static const char* routingTargetMetaAttribute(
      const MetaAttribute metaAttribute) SBE_NOEXCEPT {
    switch (metaAttribute) {
      case MetaAttribute::PRESENCE:
        return "required";
      default:
        return "";
    }
  }

  static const char* routingTargetCharacterEncoding() SBE_NOEXCEPT {
    return "UTF-8";
  }

  static SBE_CONSTEXPR std::uint64_t routingTargetSinceVersion() SBE_NOEXCEPT {
    return 0;
  }

  bool routingTargetInActingVersion() SBE_NOEXCEPT { return true; }

  static SBE_CONSTEXPR std::uint16_t routingTargetId() SBE_NOEXCEPT {
    return 4;
  }

  static SBE_CONSTEXPR std::uint64_t routingTargetHeaderLength() SBE_NOEXCEPT {
    return 4;
  }

 private:
  void onRoutingTargetLengthAccessed() const {
    switch (codecState()) {
      case CodecState::V0_REQUESTID_DONE:
        break;
      default:
        throw AccessOrderError(
            std::string("Illegal field access order. ") +
            "Cannot decode length of var data \"routingTarget\" in state: " +
            codecStateName(codecState()) +
            ". Expected one of these transitions: [" +
            codecStateTransitions(codecState()) +
            "]. Please see the diagram in the docs of the enum LoggingContext::CodecState.");
    }
  }

 public:
  SBE_NODISCARD std::uint32_t routingTargetLength() const {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetLengthAccessed();
#endif
    std::uint32_t length;
    std::memcpy(&length, m_buffer + sbePosition(), sizeof(std::uint32_t));
    return SBE_LITTLE_ENDIAN_ENCODE_32(length);
  }

 private:
  void onRoutingTargetAccessed() {
    switch (codecState()) {
      case CodecState::V0_REQUESTID_DONE:
        codecState(CodecState::V0_ROUTINGTARGET_DONE);
        break;
      default:
        throw AccessOrderError(
            std::string("Illegal field access order. ") +
            "Cannot access field \"routingTarget\" in state: " +
            codecStateName(codecState()) +
            ". Expected one of these transitions: [" +
            codecStateTransitions(codecState()) +
            "]. Please see the diagram in the docs of the enum LoggingContext::CodecState.");
    }
  }

 public:
  std::uint64_t skipRoutingTarget() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    sbePosition(lengthPosition + lengthOfLengthField + dataLength);
    return dataLength;
  }

  SBE_NODISCARD const char* routingTarget() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + sbePosition(), sizeof(std::uint32_t));
    const char* fieldPtr = m_buffer + sbePosition() + 4;
    sbePosition(
        sbePosition() + 4 + SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue));
    return fieldPtr;
  }

  std::uint64_t getRoutingTarget(char* dst, const std::uint64_t length) {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    sbePosition(lengthPosition + lengthOfLengthField);
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    std::uint64_t bytesToCopy = length < dataLength ? length : dataLength;
    std::uint64_t pos = sbePosition();
    sbePosition(pos + dataLength);
    std::memcpy(dst, m_buffer + pos, static_cast<std::size_t>(bytesToCopy));
    return bytesToCopy;
  }

  LoggingContext& putRoutingTarget(
      const char* src, const std::uint32_t length) {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    std::uint32_t lengthFieldValue = SBE_LITTLE_ENDIAN_ENCODE_32(length);
    sbePosition(lengthPosition + lengthOfLengthField);
    std::memcpy(
        m_buffer + lengthPosition, &lengthFieldValue, sizeof(std::uint32_t));
    if (length != std::uint32_t(0)) {
      std::uint64_t pos = sbePosition();
      sbePosition(pos + length);
      std::memcpy(m_buffer + pos, src, length);
    }
    return *this;
  }

  std::string getRoutingTargetAsString() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    sbePosition(lengthPosition + lengthOfLengthField);
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    std::uint64_t pos = sbePosition();
    const std::string result(m_buffer + pos, dataLength);
    sbePosition(pos + dataLength);
    return result;
  }

  std::string getRoutingTargetAsJsonEscapedString() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::ostringstream oss;
    std::string s = getRoutingTargetAsString();

    for (const auto c : s) {
      switch (c) {
        case '"':
          oss << "\\\"";
          break;
        case '\\':
          oss << "\\\\";
          break;
        case '\b':
          oss << "\\b";
          break;
        case '\f':
          oss << "\\f";
          break;
        case '\n':
          oss << "\\n";
          break;
        case '\r':
          oss << "\\r";
          break;
        case '\t':
          oss << "\\t";
          break;

        default:
          if ('\x00' <= c && c <= '\x1f') {
            oss << "\\u" << std::hex << std::setw(4) << std::setfill('0')
                << (int)(c);
          } else {
            oss << c;
          }
      }
    }

    return oss.str();
  }

#if __cplusplus >= 201703L
  std::string_view getRoutingTargetAsStringView() {
#if defined(SBE_ENABLE_PRECEDENCE_CHECKS)
    onRoutingTargetAccessed();
#endif
    std::uint64_t lengthOfLengthField = 4;
    std::uint64_t lengthPosition = sbePosition();
    sbePosition(lengthPosition + lengthOfLengthField);
    std::uint32_t lengthFieldValue;
    std::memcpy(
        &lengthFieldValue, m_buffer + lengthPosition, sizeof(std::uint32_t));
    std::uint64_t dataLength = SBE_LITTLE_ENDIAN_ENCODE_32(lengthFieldValue);
    std::uint64_t pos = sbePosition();
    const std::string_view result(m_buffer + pos, dataLength);
    sbePosition(pos + dataLength);
    return result;
  }
#endif

  LoggingContext& putRoutingTarget(const std::string& str) {
    if (str.length() > 1073741824) {
      throw std::runtime_error("std::string too long for length type [E109]");
    }
    return putRoutingTarget(
        str.data(), static_cast<std::uint32_t>(str.length()));
  }

#if __cplusplus >= 201703L
  LoggingContext& putRoutingTarget(const std::string_view str) {
    if (str.length() > 1073741824) {
      throw std::runtime_error("std::string too long for length type [E109]");
    }
    return putRoutingTarget(
        str.data(), static_cast<std::uint32_t>(str.length()));
  }
#endif

  template <typename CharT, typename Traits>
  friend std::basic_ostream<CharT, Traits>& operator<<(
      std::basic_ostream<CharT, Traits>& builder,
      const LoggingContext& _writer) {
    LoggingContext writer(
        _writer.m_buffer,
        _writer.m_offset,
        _writer.m_bufferLength,
        _writer.m_actingBlockLength,
        _writer.m_actingVersion);

    builder << '{';
    builder << R"("Name": "LoggingContext", )";
    builder << R"("sbeTemplateId": )";
    builder << writer.sbeTemplateId();
    builder << ", ";

    builder << R"("logSampleRatio": )";
    builder << +writer.logSampleRatio();

    builder << ", ";
    builder << R"("logErrorSampleRatio": )";
    builder << +writer.logErrorSampleRatio();

    builder << ", ";
    builder << R"("requestId": )";
    builder << '"' << writer.getRequestIdAsJsonEscapedString().c_str() << '"';

    builder << ", ";
    builder << R"("routingTarget": )";
    builder << '"' << writer.getRoutingTargetAsJsonEscapedString().c_str()
            << '"';

    builder << '}';

    return builder;
  }

  void skip() {
    skipRequestId();
    skipRoutingTarget();
  }

  SBE_NODISCARD static SBE_CONSTEXPR bool isConstLength() SBE_NOEXCEPT {
    return false;
  }

  SBE_NODISCARD static std::size_t computeLength(
      std::size_t requestIdLength = 0, std::size_t routingTargetLength = 0) {
#if defined(__GNUG__) && !defined(__clang__)
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wtype-limits"
#endif
    std::size_t length = sbeBlockLength();

    length += requestIdHeaderLength();
    if (requestIdLength > 1073741824LL) {
      throw std::runtime_error(
          "requestIdLength too long for length type [E109]");
    }
    length += requestIdLength;

    length += routingTargetHeaderLength();
    if (routingTargetLength > 1073741824LL) {
      throw std::runtime_error(
          "routingTargetLength too long for length type [E109]");
    }
    length += routingTargetLength;

    return length;
#if defined(__GNUG__) && !defined(__clang__)
#pragma GCC diagnostic pop
#endif
  }
};

const std::string LoggingContext::STATE_NAME_LOOKUP[4] = {
    "NOT_WRAPPED",
    "V0_BLOCK",
    "V0_REQUESTID_DONE",
    "V0_ROUTINGTARGET_DONE",
};

const std::string LoggingContext::STATE_TRANSITIONS_LOOKUP[4] = {
    "\"wrap(version=0)\"",
    "\"logSampleRatio(?)\", \"logErrorSampleRatio(?)\", \"requestIdLength()\", \"requestId(?)\"",
    "\"routingTargetLength()\", \"routingTarget(?)\"",
    "",
};

} // namespace sbe
} // namespace thrift
} // namespace apache
#endif
